@@ -2,11 +2,10 @@
 <?php
 /*
  +-----------------------------------------------------------------------+
- | bin/initdb.sh                                                         |
- |                                                                       |
  | This file is part of the Roundcube Webmail client                     |
- | Copyright (C) 2010-2015, The Roundcube Dev Team                       |
- | Copyright (C) 2010-2015, Kolab Systems AG                             |
+ |                                                                       |
+ | Copyright (C) The Roundcube Dev Team                                  |
+ | Copyright (C) Kolab Systems AG                                        |
  |                                                                       |
  | Licensed under the GNU General Public License version 3 or            |
  | any later version with exceptions for skins & plugins.                |
@@ -38,5 +37,3 @@ if (!file_exists($opts['dir'])) {
 }
 
 rcmail_utils::db_init($opts['dir']);
-
-?>
