@@ -2,14 +2,19 @@
 
 /**
  +-----------------------------------------------------------------------+
- | rcmail_install.php                                                    |
+ | This file is part of the Roundcube Webmail client                     |
  |                                                                       |
- | This file is part of the Roundcube Webmail package                    |
- | Copyright (C) 2008-2016, The Roundcube Dev Team                       |
+ | Copyright (C) The Roundcube Dev Team                                  |
  |                                                                       |
  | Licensed under the GNU General Public License version 3 or            |
  | any later version with exceptions for skins & plugins.                |
  | See the README file for a full license statement.                     |
+ |                                                                       |
+ | PURPOSE:                                                              |
+ |   Roundcube Installer                                                 |
+ +-----------------------------------------------------------------------+
+ | Author: Thomas Bruederli <roundcube@gmail.com>                        |
+ | Author: Aleksander Machniak <alec@alec.pl>                            |
  +-----------------------------------------------------------------------+
 */
 
@@ -17,8 +22,7 @@
  * Class to control the installation process of the Roundcube Webmail package
  *
  * @category Install
- * @package  Roundcube
- * @author   Thomas Bruederli
+ * @package  Webmail
  */
 class rcmail_install
 {
@@ -33,7 +37,7 @@ class rcmail_install
     public $bool_config_props = array();
 
     public $local_config    = array('db_dsnw', 'default_host', 'support_url', 'des_key', 'plugins');
-    public $obsolete_config = array('db_backend', 'db_max_length', 'double_auth', 'preview_pane');
+    public $obsolete_config = array('db_backend', 'db_max_length', 'double_auth', 'preview_pane', 'debug_level', 'referer_check');
     public $replaced_config = array(
         'skin_path'            => 'skin',
         'locale_string'        => 'language',
@@ -57,6 +61,11 @@ class rcmail_install
         'Oracle'              => 'oci8',
     );
 
+    /** @var array List of config options with default value change per-release */
+    public $defaults_changes = array(
+        '1.4.0' => array('skin', 'smtp_port', 'smtp_user', 'smtp_pass'),
+        '1.4.1' => array('jquery_ui_skin_map'),
+    );
 
     /**
      * Constructor
@@ -206,16 +215,7 @@ class rcmail_install
             }
 
             // convert some form data
-            if ($prop == 'debug_level' && !$is_default) {
-                if (is_array($value)) {
-                    $val = 0;
-                    foreach ($value as $dbgval) {
-                        $val += intval($dbgval);
-                    }
-                    $value = $val;
-                }
-            }
-            else if ($prop == 'db_dsnw' && !empty($_POST['_dbtype'])) {
+            if ($prop == 'db_dsnw' && !empty($_POST['_dbtype'])) {
                 if ($_POST['_dbtype'] == 'sqlite') {
                     $value = sprintf('%s://%s?mode=0646', $_POST['_dbtype'],
                         $_POST['_dbname'][0] == '/' ? '/' . $_POST['_dbname'] : $_POST['_dbname']);
@@ -303,9 +303,11 @@ class rcmail_install
      * Check the current configuration for missing properties
      * and deprecated or obsolete settings
      *
+     * @param string $version Previous version on upgrade
+     *
      * @return array List with problems detected
      */
-    public function check_config()
+    public function check_config($version = null)
     {
         $this->load_config();
 
@@ -384,6 +386,18 @@ class rcmail_install
             }
         }
 
+        if ($version) {
+            $out['defaults'] = array();
+
+            foreach ($this->defaults_changes as $v => $opts) {
+                if (version_compare($v, $version, '>')) {
+                    $out['defaults'] = array_merge($out['defaults'], $opts);
+                }
+            }
+
+            $out['defaults'] = array_unique($out['defaults']);
+        }
+
         return $out;
     }
 
@@ -449,9 +463,23 @@ class rcmail_install
 
         // read reference schema from mysql.initial.sql
         $engine    = $db->db_provider;
-        $db_schema = $this->db_read_schema(INSTALL_PATH . "SQL/$engine.initial.sql");
+        $db_schema = $this->db_read_schema(INSTALL_PATH . "SQL/$engine.initial.sql", $schema_version);
         $errors    = array();
 
+        // Just check the version
+        if ($schema_version) {
+            $version = rcmail_utils::db_version();
+
+            if (empty($version)) {
+                $errors[] = "Schema version not found";
+            }
+            else if ($schema_version != $version) {
+                $errors[] = "Schema version: {$version} (required: {$schema_version})";
+            }
+
+            return !empty($errors) ? $errors : false;
+        }
+
         // check list of tables
         $existing_tables = $db->list_tables();
 
@@ -466,7 +494,7 @@ class rcmail_install
                 $diff    = array_diff(array_keys($cols), $db_cols);
 
                 if (!empty($diff)) {
-                    $errors[] = "Missing columns in table '$table': " . join(',', $diff);
+                    $errors[] = "Missing columns in table '$table': " . implode(',', $diff);
                 }
             }
         }
@@ -477,12 +505,11 @@ class rcmail_install
     /**
      * Utility function to read database schema from an .sql file
      */
-    private function db_read_schema($schemafile)
+    private function db_read_schema($schemafile, &$version = null)
     {
-        $lines       = file($schemafile);
-        $table_block = false;
-        $schema      = array();
-        $keywords    = array('PRIMARY','KEY','INDEX','UNIQUE','CONSTRAINT','REFERENCES','FOREIGN');
+        $lines    = file($schemafile);
+        $schema   = array();
+        $keywords = array('PRIMARY','KEY','INDEX','UNIQUE','CONSTRAINT','REFERENCES','FOREIGN');
 
         foreach ($lines as $line) {
             if (preg_match('/^\s*create table ([\S]+)/i', $line, $m)) {
@@ -490,6 +517,9 @@ class rcmail_install
                 $table_name = end($table_name);
                 $table_name = preg_replace('/[`"\[\]]/', '', $table_name);
             }
+            else if (preg_match('/insert into/i', $line) && preg_match('/\'roundcube-version\',\s*\'([0-9]+)\'/', $line, $m)) {
+                $version = $m[1];
+            }
             else if ($table_name && ($line = trim($line))) {
                 if ($line == 'GO' || $line[0] == ')' || $line[strlen($line)-1] == ';') {
                     $table_name = null;
@@ -787,7 +817,7 @@ class rcmail_install
                 }
 
                 if ($isnum) {
-                    return 'array(' . join(', ', array_map(array('rcmail_install', '_dump_var'), $var)) . ')';
+                    return 'array(' . implode(', ', array_map(array('rcmail_install', '_dump_var'), $var)) . ')';
                 }
             }
         }
