@@ -6,7 +6,7 @@
  * @licstart  The following is the entire license notice for the
  * JavaScript code in this file.
  *
- * Copyright (c) 2005-2014, The Roundcube Dev Team
+ * Copyright (c) The Roundcube Dev Team
  *
  * The JavaScript code in this page is free software: you can
  * redistribute it and/or modify it under the terms of the GNU
@@ -58,26 +58,22 @@ function roundcube_browser()
   this.dom = document.getElementById ? true : false;
   this.dom2 = document.addEventListener && document.removeEventListener;
 
-  this.webkit = this.agent_lc.indexOf('applewebkit') > 0;
+  this.edge = this.agent_lc.indexOf(' edge/') > 0;
+  this.webkit = !this.edge && this.agent_lc.indexOf('applewebkit') > 0;
   this.ie = (document.all && !window.opera) || (this.win && this.agent_lc.indexOf('trident/') > 0);
 
-  if (this.ie) {
-    this.ie7 = n.appVersion.indexOf('MSIE 7') > 0;
-    this.ie8 = n.appVersion.indexOf('MSIE 8') > 0;
-    this.ie9 = n.appVersion.indexOf('MSIE 9') > 0;
-  }
-  else if (window.opera) {
+  if (window.opera) {
     this.opera = true; // Opera < 15
     this.vendver = opera.version();
   }
-  else {
+  else if (!this.ie && !this.edge) {
     this.chrome = this.agent_lc.indexOf('chrome') > 0;
     this.opera = this.webkit && this.agent.indexOf(' OPR/') > 0; // Opera >= 15
     this.safari = !this.chrome && !this.opera && (this.webkit || this.agent_lc.indexOf('safari') > 0);
     this.konq = this.agent_lc.indexOf('konqueror') > 0;
     this.mz = this.dom && !this.chrome && !this.safari && !this.konq && !this.opera && this.agent.indexOf('Mozilla') >= 0;
-    this.iphone = this.safari && (this.agent_lc.indexOf('iphone') > 0 || this.agent_lc.indexOf('ipod') > 0);
-    this.ipad = this.safari && this.agent_lc.indexOf('ipad') > 0;
+    this.iphone = this.safari && (this.agent_lc.indexOf('iphone') > 0 || this.agent_lc.indexOf('ipod') > 0 || this.platform == 'ipod' || this.platform == 'iphone');
+    this.ipad = this.safari && (this.agent_lc.indexOf('ipad') > 0 || this.platform == 'ipad');
   }
 
   if (!this.vendver) {
@@ -93,9 +89,10 @@ function roundcube_browser()
   if (this.safari && (/;\s+([a-z]{2})-[a-z]{2}\)/.test(this.agent_lc)))
     this.lang = RegExp.$1;
 
-  this.tablet = /ipad|android|xoom|sch-i800|playbook|tablet|kindle/i.test(this.agent_lc);
   this.mobile = /iphone|ipod|blackberry|iemobile|opera mini|opera mobi|mobile/i.test(this.agent_lc);
+  this.tablet = !this.mobile && /ipad|android|xoom|sch-i800|playbook|tablet|kindle/i.test(this.agent_lc);
   this.touch = this.mobile || this.tablet;
+  this.pointer = typeof window.PointerEvent == "function";
   this.cookies = n.cookieEnabled;
 
   // test for XMLHTTP support
@@ -113,7 +110,9 @@ function roundcube_browser()
     var classname = ' js';
 
     if (this.ie)
-      classname += ' ie ie'+parseInt(this.vendver);
+      classname += ' ms ie ie'+parseInt(this.vendver);
+    else if (this.edge)
+      classname += ' ms edge';
     else if (this.opera)
       classname += ' opera';
     else if (this.konq)
@@ -152,7 +151,7 @@ var rcube_event = {
 get_target: function(e)
 {
   e = e || window.event;
-  return e && e.target ? e.target : e.srcElement;
+  return e && e.target ? e.target : e.srcElement || document;
 },
 
 /**
@@ -280,17 +279,21 @@ cancel: function(evt)
 },
 
 /**
- * Determine whether the given event was trigered from keyboard
+ * Determine whether the given event was triggered from keyboard
  */
 is_keyboard: function(e)
 {
-  return e && (
-      (e.pointerType !== undefined && e.pointerType !== 'mouse') ||       // IE 11+
-      (e.mozInputSource && e.mozInputSource == e.MOZ_SOURCE_KEYBOARD) ||  // Firefox
-      (e.offsetX === 0 && e.offsetY === 0) || // Opera
-      (!e.pageX && (e.pageY || 0) <= 0 && !e.clientX && (e.clientY || 0) <= 0) ||  // others
-      (bw.ie && rcube_event._last_keyboard_event && rcube_event._last_keyboard_event.target == e.target)  // hack for IE <= 10
-    );
+  if (!e)
+    return false;
+
+  // DOM3-compatible
+  // An event invoked by pressing Enter on a link will produce a 'click' event,
+  // so we have to extend the check, e.g. with use of e.clientX.
+  if (e.type)
+    return !!e.type.match(/^key/) || (e.type == 'click' && !e.clientX);
+
+  // Old browsers
+  return !e.pageX && (e.pageY || 0) <= 0 && !e.clientX && (e.clientY || 0) <= 0;
 },
 
 /**
@@ -361,7 +364,10 @@ removeEventListener: function(evt, func, obj)
  */
 triggerEvent: function(evt, e)
 {
-  var ret, h;
+  var ret, h,
+    reset_fn = function(o) {
+      try { if (o && o.event) delete o.event; } catch(err) { };
+    };
 
   if (e === undefined)
     e = this;
@@ -385,26 +391,11 @@ triggerEvent: function(evt, e)
           break;
       }
     }
-    if (ret && ret.event) {
-      try {
-        delete ret.event;
-      } catch (err) {
-        // IE6-7 doesn't support deleting HTMLFormElement attributes (#1488017)
-        $(ret).removeAttr('event');
-      }
-    }
+    reset_fn(ret);
   }
 
   delete this._event_exec[evt];
-
-  if (e.event) {
-    try {
-      delete e.event;
-    } catch (err) {
-      // IE6-7 doesn't support deleting HTMLFormElement attributes (#1488017)
-      $(e).removeAttr('event');
-    }
-  }
+  reset_fn(e);
 
   return ret;
 }
@@ -415,10 +406,14 @@ triggerEvent: function(evt, e)
 // check if input is a valid email address
 // By Cal Henderson <cal@iamcal.com>
 // http://code.iamcal.com/php/rfc822/
-function rcube_check_email(input, inline)
+function rcube_check_email(input, inline, count, strict)
 {
-  if (input && window.RegExp) {
-    var qtext = '[^\\x0d\\x22\\x5c\\x80-\\xff]',
+  if (!input)
+    return count ? 0 : false;
+
+  if (count) inline = true;
+
+  var qtext = '[^\\x0d\\x22\\x5c\\x80-\\xff]',
       dtext = '[^\\x0d\\x5b-\\x5d\\x80-\\xff]',
       atom = '[^\\x00-\\x20\\x22\\x28\\x29\\x2c\\x2e\\x3a-\\x3c\\x3e\\x40\\x5b-\\x5d\\x7f-\\xff]+',
       quoted_pair = '\\x5c[\\x00-\\x7f]',
@@ -447,16 +442,17 @@ function rcube_check_email(input, inline)
         '\\u05d1\\u05f2\\u05b7\\u05e9\\u05e4\\u05bc\\u05d9\\u05dc\\x2e\\u05d8\\u05e2\\u05e1\\u05d8'
       ],
       icann_addr = 'mailtest\\x40('+icann_domains.join('|')+')',
-      word = '('+atom+'|'+quoted_string+')',
-      delim = '[,;\s\n]',
+      word = strict ? '('+atom+'|'+quoted_string+')' : '[^\\u0000-\\u0020\\u002e\\u00a0\\u0040\\u007f\\u2028\\u2029]+',
+      delim = '[,;\\s\\n]',
       local_part = word+'(\\x2e'+word+')*',
       addr_spec = '(('+local_part+'\\x40'+domain+')|('+icann_addr+'))',
-      reg1 = inline ? new RegExp('(^|<|'+delim+')'+addr_spec+'($|>|'+delim+')', 'i') : new RegExp('^'+addr_spec+'$', 'i');
+      rx_flag = count ? 'ig' : 'i',
+      rx = inline ? new RegExp('(^|<|'+delim+')'+addr_spec+'($|>|'+delim+')', rx_flag) : new RegExp('^'+addr_spec+'$', 'i');
 
-    return reg1.test(input) ? true : false;
-  }
+  if (count)
+    return input.match(rx).length;
 
-  return false;
+  return rx.test(input);
 };
 
 // recursively copy an object
@@ -613,49 +609,32 @@ if (!String.prototype.startsWith) {
   };
 }
 
+if (!String.prototype.endsWith) {
+  String.prototype.endsWith = function(searchString, position) {
+    var subjectString = this.toString();
+    if (typeof position !== 'number' || !isFinite(position) || Math.floor(position) !== position || position > subjectString.length) {
+      position = subjectString.length;
+    }
+    position -= searchString.length;
+    var lastIndex = subjectString.lastIndexOf(searchString, position);
+    return lastIndex !== -1 && lastIndex === position;
+  };
+}
+
 // array utility function
 jQuery.last = function(arr) {
   return arr && arr.length ? arr[arr.length-1] : undefined;
 }
 
-// jQuery plugin to emulate HTML5 placeholder attributes on input elements
+// jQuery plugin to set HTML5 placeholder and title attributes on input elements
 jQuery.fn.placeholder = function(text) {
   return this.each(function() {
-    var active = false, elem = $(this);
-    this.title = text;
-
-    // Try HTML5 placeholder attribute first
-    if ('placeholder' in this) {
-      elem.attr('placeholder', text);
-    }
-    // Fallback to Javascript emulation of placeholder
-    else {
-      this._placeholder = text;
-      elem.blur(function(e) {
-        if ($.trim(elem.val()) == "")
-          elem.val(text);
-        elem.triggerHandler('change');
-      })
-      .focus(function(e) {
-        if ($.trim(elem.val()) == text)
-          elem.val("");
-        elem.triggerHandler('change');
-      })
-      .change(function(e) {
-        var active = elem.val() == text;
-        elem[(active ? 'addClass' : 'removeClass')]('placeholder').attr('spellcheck', active);
-      });
-
-      // Do not blur currently focused element (catch exception: #1489008)
-      try { active = this == document.activeElement; } catch(e) {}
-      if (!active)
-        elem.blur();
-    }
+    $(this).prop({title: text, placeholder: text});
   });
 };
 
 // function to parse query string into an object
-rcube_parse_query = function(query)
+var rcube_parse_query = function(query)
 {
   if (!query)
     return {};
@@ -714,7 +693,7 @@ var Base64 = (function () {
 
   // private method for UTF-8 decoding
   var utf8_decode = function (utftext) {
-    var i = 0, string = '', c = c2 = c3 = 0;
+    var i = 0, string = '', c = 0, c2 = 0, c3 = 0;
 
     while (i < utftext.length) {
       c = utftext.charCodeAt(i);
@@ -744,16 +723,16 @@ var Base64 = (function () {
      * @param {String} input The string to encode in base64.
      */
     encode: function (input) {
+      // encode UTF8 as btoa() may fail on some characters
+      input = utf8_encode(input);
+
       if (typeof(window.btoa) === 'function') {
-        // it may fail on unicode characters, the fallback can handle them
         try {
           return btoa(input);
         }
         catch (e) {};
       }
 
-      input = utf8_encode(input);
-
       var chr1, chr2, chr3, enc1, enc2, enc3, enc4, i = 0, output = '', len = input.length;
 
       while (i < len) {
@@ -785,7 +764,6 @@ var Base64 = (function () {
      */
     decode: function (input) {
       if (typeof(window.atob) === 'function') {
-        // it may fail on unicode characters, the fallback can handle them
         try {
           return utf8_decode(atob(input));
         }
