@@ -1,9 +1,10 @@
 <?php
 
-/*
+/**
  +-----------------------------------------------------------------------+
  | This file is part of the Roundcube Webmail client                     |
- | Copyright (C) 2008-2012, The Roundcube Dev Team                       |
+ |                                                                       |
+ | Copyright (C) The Roundcube Dev Team                                  |
  |                                                                       |
  | Licensed under the GNU General Public License version 3 or            |
  | any later version with exceptions for skins & plugins.                |
@@ -16,7 +17,7 @@
  +-----------------------------------------------------------------------+
 */
 
-// location where plugins are loade from
+// location where plugins are loaded from
 if (!defined('RCUBE_PLUGINS_DIR')) {
     define('RCUBE_PLUGINS_DIR', RCUBE_INSTALL_PATH . 'plugins/');
 }
@@ -34,6 +35,8 @@ class rcube_plugin_api
     public $dir;
     public $url = 'plugins/';
     public $task = '';
+    public $initialized = false;
+
     public $output;
     public $handlers              = array();
     public $allowed_prefs         = array();
@@ -41,6 +44,7 @@ class rcube_plugin_api
     public $active_plugins        = array();
 
     protected $plugins           = array();
+    protected $plugins_initialized = array();
     protected $tasks             = array();
     protected $actions           = array();
     protected $actionmap         = array();
@@ -85,12 +89,21 @@ class rcube_plugin_api
     {
         $this->task   = $task;
         $this->output = $app->output;
-
         // register an internal hook
         $this->register_hook('template_container', array($this, 'template_container_hook'));
-
         // maybe also register a shudown function which triggers
         // shutdown functions of all plugin objects
+
+        foreach ($this->plugins as $plugin) {
+            // ... task, request type and framed mode
+            if (!$this->plugins_initialized[$plugin->ID] && !$this->filter($plugin)) {
+                $plugin->init();
+                $this->plugins_initialized[$plugin->ID] = $plugin;
+            }
+        }
+
+        // we have finished initializing all plugins
+        $this->initialized = true;
     }
 
     /**
@@ -136,7 +149,7 @@ class rcube_plugin_api
     /**
      * Load the specified plugin
      *
-     * @param string Plugin name
+     * @param string  Plugin name
      * @param boolean Force loading of the plugin even if it doesn't match the filter
      * @param boolean Require loading of the plugin, error if it doesn't exist
      *
@@ -151,55 +164,83 @@ class rcube_plugin_api
             $plugins_dir = unslashify($dir->path);
         }
 
-        // plugin already loaded
-        if ($this->plugins[$plugin_name]) {
-            return true;
+        // Validate the plugin name to prevent from path traversal
+        if (preg_match('/[^a-zA-Z0-9_-]/', $plugin_name)) {
+            rcube::raise_error(array('code' => 520,
+                    'file' => __FILE__, 'line' => __LINE__,
+                    'message' => "Invalid plugin name: $plugin_name"), true, false);
+            return false;
         }
 
-        $fn = "$plugins_dir/$plugin_name/$plugin_name.php";
+        // plugin already loaded?
+        if (!$this->plugins[$plugin_name]) {
+            $fn = "$plugins_dir/$plugin_name/$plugin_name.php";
+
+            if (!is_readable($fn)) {
+                if ($require) {
+                    rcube::raise_error(array('code' => 520, 'type' => 'php',
+                        'file' => __FILE__, 'line' => __LINE__,
+                        'message' => "Failed to load plugin file $fn"), true, false);
+                }
+
+                return false;
+            }
 
-        if (is_readable($fn)) {
             if (!class_exists($plugin_name, false)) {
                 include $fn;
             }
 
             // instantiate class if exists
-            if (class_exists($plugin_name, false)) {
-                $plugin = new $plugin_name($this);
-                $this->active_plugins[] = $plugin_name;
-
-                // check inheritance...
-                if (is_subclass_of($plugin, 'rcube_plugin')) {
-                    // ... task, request type and framed mode
-                    if (($force || !$plugin->task || preg_match('/^('.$plugin->task.')$/i', $this->task))
-                        && (!$plugin->noajax || (is_object($this->output) && $this->output->type == 'html'))
-                        && (!$plugin->noframe || empty($_REQUEST['_framed']))
-                    ) {
-                        $plugin->init();
-                        $this->plugins[$plugin_name] = $plugin;
-                    }
-
-                    if (!empty($plugin->allowed_prefs)) {
-                        $this->allowed_prefs = array_merge($this->allowed_prefs, $plugin->allowed_prefs);
-                    }
-
-                    return true;
-                }
-            }
-            else {
+            if (!class_exists($plugin_name, false)) {
                 rcube::raise_error(array('code' => 520, 'type' => 'php',
                     'file' => __FILE__, 'line' => __LINE__,
                     'message' => "No plugin class $plugin_name found in $fn"),
                     true, false);
+
+                return false;
+            }
+
+            $plugin = new $plugin_name($this);
+            $this->active_plugins[] = $plugin_name;
+
+            // check inheritance...
+            if (is_subclass_of($plugin, 'rcube_plugin')) {
+                // call onload method on plugin if it exists.
+                // this is useful if you want to be called early in the boot process
+                if (method_exists($plugin, 'onload')) {
+                    $plugin->onload();
+                }
+
+                if (!empty($plugin->allowed_prefs)) {
+                    $this->allowed_prefs = array_merge($this->allowed_prefs, $plugin->allowed_prefs);
+                }
+
+                $this->plugins[$plugin_name] = $plugin;
             }
         }
-        else if ($require) {
-            rcube::raise_error(array('code' => 520, 'type' => 'php',
-                'file' => __FILE__, 'line' => __LINE__,
-                'message' => "Failed to load plugin file $fn"), true, false);
+
+        if ($plugin = $this->plugins[$plugin_name]) {
+            // init a plugin only if $force is set or if we're called after initialization
+            if (($force || $this->initialized) && !$this->plugins_initialized[$plugin_name] && ($force || !$this->filter($plugin))) {
+                $plugin->init();
+                $this->plugins_initialized[$plugin_name] = $plugin;
+            }
         }
 
-        return false;
+        return true;
+    }
+
+    /**
+     * check if we should prevent this plugin from initialising
+     *
+     * @param $plugin
+     * @return bool
+     */
+    private function filter($plugin)
+    {
+        return ($plugin->noajax  && !(is_object($this->output) && $this->output->type == 'html'))
+             || ($plugin->task && !preg_match('/^('.$plugin->task.')$/i', $this->task))
+             || ($plugin->noframe && !empty($_REQUEST['_framed']));
     }
 
     /**
@@ -250,6 +291,14 @@ class rcube_plugin_api
         $fn   = unslashify($dir->path) . "/$plugin_name/$plugin_name.php";
         $info = false;
 
+        // Validate the plugin name to prevent from path traversal
+        if (preg_match('/[^a-zA-Z0-9_-]/', $plugin_name)) {
+            rcube::raise_error(array('code' => 520,
+                    'file' => __FILE__, 'line' => __LINE__,
+                    'message' => "Invalid plugin name: $plugin_name"), true, false);
+            return false;
+        }
+
         if (!class_exists($plugin_name, false)) {
             if (is_readable($fn)) {
                 include($fn);
@@ -293,8 +342,8 @@ class rcube_plugin_api
             // load additional information from local composer.lock file
             if ($lock = $composer_lock['installed'][$json['name']]) {
                 $info['version'] = $lock['version'];
-                $info['uri']     = $lock['homepage'] ? $lock['homepage'] : $lock['source']['uri'];
-                $info['src_uri'] = $lock['dist']['uri'] ? $lock['dist']['uri'] : $lock['source']['uri'];
+                $info['uri']     = $lock['homepage'] ?: $lock['source']['uri'];
+                $info['src_uri'] = $lock['dist']['uri'] ?: $lock['source']['uri'];
             }
         }
 
@@ -403,14 +452,16 @@ class rcube_plugin_api
         array_push($this->exec_stack, $hook);
 
         // Use for loop here, so handlers added in the hook will be executed too
-        for ($i = 0; $i < count($this->handlers[$hook]); $i++) {
-            $ret = call_user_func($this->handlers[$hook][$i], $args);
-            if ($ret && is_array($ret)) {
-                $args = $ret + $args;
-            }
+        if (!empty($this->handlers[$hook])) {
+            for ($i = 0; $i < count($this->handlers[$hook]); $i++) {
+                $ret = call_user_func($this->handlers[$hook][$i], $args);
+                if ($ret && is_array($ret)) {
+                    $args = $ret + $args;
+                }
 
-            if ($args['break']) {
-                break;
+                if ($args['break']) {
+                    break;
+                }
             }
         }
 
@@ -562,8 +613,7 @@ class rcube_plugin_api
     {
         if (is_object($this->output) && $this->output->type == 'html') {
             $src = $this->resource_url($fn);
-            $this->output->add_header(html::tag('script',
-                array('type' => "text/javascript", 'src' => $src)));
+            $this->output->include_script($src, 'head_bottom', false);
         }
     }
 
@@ -575,6 +625,31 @@ class rcube_plugin_api
     public function include_stylesheet($fn)
     {
         if (is_object($this->output) && $this->output->type == 'html') {
+            if ($fn[0] != '/' && !preg_match('|^https?://|i', $fn)) {
+                $rcube      = rcube::get_instance();
+                $devel_mode = $rcube->config->get('devel_mode');
+                $assets_dir = $rcube->config->get('assets_dir');
+                $path       = unslashify($assets_dir ?: RCUBE_INSTALL_PATH);
+
+                // Prefer .less files in devel_mode (assume less.js is loaded)
+                if ($devel_mode) {
+                    $less = preg_replace('/\.css$/i', '.less', $fn);
+                    if ($less != $fn && is_file("$path/plugins/$less")) {
+                        $fn = $less;
+                    }
+                }
+                else if (!preg_match('/\.min\.css$/', $fn)) {
+                    $min = preg_replace('/\.css$/i', '.min.css', $fn);
+                    if (is_file("$path/plugins/$min")) {
+                        $fn = $min;
+                    }
+                }
+
+                if (!is_file("$path/plugins/$fn")) {
+                    return;
+                }
+            }
+
             $src = $this->resource_url($fn);
             $this->output->include_css($src);
         }
